/*
    OTP4U 0.8.0 - One Time Pad for you
    Copyright (C) 2003 Pierre Blanc
    Pierre Blanc: blanc_teutoburgo@yahoo.it
    http://www.teutoburgo.tk
    http://it.geocities.com/teutoburgo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    or go to      http://www.gnu.org/copyleft/gpl.html
*/

package tk.teutoburgo.otp4u.crypt;

import tk.teutoburgo.otp4u.constants.*;
import tk.teutoburgo.otp4u.io.*;
import tk.teutoburgo.otp4u.util.*;
import java.io.*;

/**
 * Title:        OTP4U
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:      Teutoburgo
 * @author Pierre Blanc
 * @version 1.0
 */

public class Cipher {
  public static final String RANDOM_KEY_FILE=Constants.RANDOM_KEY_FILE,
		PLAIN=Constants.PLAIN, CIPHER=Constants.CIPHER,
		RANDOM_KEY=Constants.RANDOM_KEY;
  public String OTP4U_HOME="";
  Xor xor = new Xor();
  Io io ;
  boolean isGui=false;

  MessagesPrinter mp;

  public Cipher() {
  }
  public Cipher(String home, boolean isGui) {
    OTP4U_HOME=home;
    this.isGui=isGui;
    mp = new MessagesPrinter (isGui);
    io= new Io(home, isGui);
  }

    /**
     * Performs decryption if the files of ciphertext and randomKey are found <br>
     * Writes the plaintext array in its directory
     * @param pathName The pathName of the ciphertext
     */
  public int decipher(String pathName){
		File f=new File(pathName);

		if(!f.isAbsolute())
			pathName=(new File(OTP4U_HOME+CIPHER+pathName)).getAbsolutePath();
		String fileName=f.getName();

		try{
			byte[][] arrays = io.initArrays(pathName);

                  byte[] ciphertext = arrays[0];
                  byte[] randomKeyBytes = arrays[1];
                  int l=ciphertext.length;
                  mp.sPrintLog("Starting decryption of file "+pathName);

                  if (l < randomKeyBytes.length){
                          byte[] realKey = getRealKey(randomKeyBytes, l);
                          byte[] plaintext = xor.decrypt(ciphertext, realKey);
			  io.writeFile(OTP4U_HOME + PLAIN+fileName, plaintext);
                  }
                  else {
                          mp.sPrintLog("Error - Key too short");
                          return -2;
                  }
		} catch (IOException e){
		  e.printStackTrace();
		  mp.sCatchException(e.getMessage());
		} catch (Exception e){
		  mp.sCatchException(e.getMessage());
		  return -1;
		}

        return 0;
  }
    /**
     * Performs encryption if the files of plaintext and randomKey are found <br>
     * Writes the ciphertext array ciphertext in its directory
     * @param pathName The pathname of the plaintext
     */
    public void encipher(String pathName) {
		int i, j, l, pr = 0,  cLength = 0, rLen;
		byte[] plaintext, ciphertext=null, realKey;
		File f=new File(pathName);
		if(!f.isAbsolute())
			pathName=(new File(OTP4U_HOME+PLAIN+pathName)).getAbsolutePath();
		String fileName=f.getName();
		mp.sPrintLog("Starting encryption of file "+pathName);
		try{
		  byte[][] arrays = io.initArrays(pathName);
		  plaintext = arrays[0];
		  byte[] randomKeyBytes = arrays[1];
		  l=plaintext.length;
		  rLen=randomKeyBytes.length;
		  if (l > rLen)
			  mp.sPrintLog("Error - The plaintext is too big.. expand the randomKey"+
					   " directory");
		  else{
			  realKey = this.getRealKey(randomKeyBytes, l);
			  ciphertext = xor.encrypt(plaintext, realKey);
			  io.writeFile(OTP4U_HOME + CIPHER+fileName, ciphertext);
		  }
		} catch (IOException e){
		  e.printStackTrace();
		  mp.sCatchException(e.getMessage());
		} catch (Exception e){
		   e.printStackTrace();
		  mp.sCatchException(e.getMessage());
		}
    }

    /**
     * Gets from the randomKey file the exact number of bytes needed for a
	 * particular encryption/decryption, then removes them from randomKey <br>
	 * @param random the randomKey array
	 * @param l the number of bytes needed
	 */
    public byte[] getRealKey(byte[] random, int l){
      int i,rLen = random.length;
      byte[] realKey=new byte[l];

      if (l>rLen) return null;
	  byte[] newRandom = new byte[rLen-l];
      for(i=0; i<l; i++){
		realKey[i]=random[i];
      }
      for(i=l; i<rLen; i++)
		newRandom[i-l]=random[i];
	  try{
		  io.writeFile(OTP4U_HOME + RANDOM_KEY+RANDOM_KEY_FILE, newRandom);
      }catch (Exception e){
        e.printStackTrace();
      }

      return realKey;
    }

}